-- +----------------------------------------------------------------------+
-- |                         Jordi Pascual Aran  (JPL)                    |
-- |                         DBA UOC                                      |
-- |----------------------------------------------------------------------|
-- | DATABASE : Oracle                                                    |
-- | FILE     : PKG_APP.sql                                               |
-- | CLASS    : Install                                                   |
-- | PURPOSE  : Procediments ABM_APLICACION                               |
-- | DATE     : 27/11/2012                                                |
-- | VERSION  : Oracle 10g                                                |
-- |----------------------------------------------------------------------|
-- | USAGE:    sqlplus UOC/UOC @PKG_APP.sql                               |
-- |----------------------------------------------------------------------|

set serveroutput on;

/* Capalera del paquet */
CREATE OR REPLACE PACKAGE PKG_APP AS

FUNCTION consulta_APP (CODI_APP IN VARCHAR2) RETURN BOOLEAN;

FUNCTION consulta_VERSIO (in_CODI_APP IN VARCHAR2, in_VERSION IN VARCHAR2) RETURN BOOLEAN;

PROCEDURE alta_APP (in_APP_ID IN VARCHAR2, in_VERSION IN VARCHAR2, in_SO_NOMBRE IN VARCHAR2, in_PAIS_ID IN CHAR, in_DESA_ID IN VARCHAR2, in_RES_VALOR IN VARCHAR2, in_APP_ACTIVA IN CHAR, in_APP_PRECIO IN NUMBER, in_APP_DESCRIPCION IN VARCHAR2, in_APP_PESO IN NUMBER, in_APP_VIDEO IN VARCHAR2, in_APP_FICHERO IN VARCHAR2, RSP OUT VARCHAR2);

PROCEDURE baixa_APP (CODI_APP IN VARCHAR2, RSP OUT VARCHAR2); 

PROCEDURE baixa_VERSIO (in_CODI_APP IN VARCHAR2, in_VERSION IN VARCHAR2, RSP OUT VARCHAR2); 

PROCEDURE modif_APP (in_APP_ID IN VARCHAR2, in_VERSION IN VARCHAR2, in_SO_NOMBRE IN VARCHAR2, in_PAIS_ID IN CHAR, in_DESA_ID IN VARCHAR2, in_RES_VALOR IN VARCHAR2, in_APP_ACTIVA IN CHAR, in_APP_PRECIO IN NUMBER, in_APP_DESCRIPCION IN VARCHAR2, in_APP_PESO IN NUMBER, in_APP_VIDEO IN VARCHAR2, in_APP_FICHERO IN VARCHAR2, RSP OUT VARCHAR2);

APP UOC.APLICACION%ROWTYPE;

END PKG_APP;
/

/* Cos del paquet */
CREATE OR REPLACE PACKAGE BODY PKG_APP AS

/* Procediment que treu per pantalla les dades d'un usuari */
PROCEDURE mostra_APP
IS
BEGIN
  DBMS_OUTPUT.put_line('Aplicaci: ' || APP.APP_ID);
  DBMS_OUTPUT.put_line('Versi: ' || APP.VERSION);
  DBMS_OUTPUT.put_line('Sistema Operatiu de la versi: ' || APP.SO_NOMBRE);
  DBMS_OUTPUT.put_line('Codi desenvolupador: ' || APP.DESA_ID);
  DBMS_OUTPUT.put_line('Preu: ' || APP.APP_PRECIO);
  DBMS_OUTPUT.put_line('Valor de resoluci: ' || APP.RES_VALOR);
  DBMS_OUTPUT.put_line('Aplicaci activa o inactiva: ' || APP.APP_ACTIVA);
  DBMS_OUTPUT.put_line('Data d''alta: ' || APP.FECHA_SUBIDA);
  DBMS_OUTPUT.put_line('Mida Aplicaci en Kbits: ' || APP.APP_PESO);
  DBMS_OUTPUT.put_line('Enlla a video demo: ' || APP.APP_VIDEO);
END;

/* Procediment compta el nmero total d'aplicacions donades d'alta */
PROCEDURE compta_APP
IS
  num number;
BEGIN
  /* Compte el nmero de usuaris donats d'alta */
  SELECT COUNT(DISTINCT APP_ID) INTO num FROM UOC.APLICACION;
  /* Mostra el resultat per pantalla */
  DBMS_OUTPUT.put_line('Actualment hi han ' || num || ' aplicacions donades d''alta.');
END ;

/* ------------------------------------------------------- */
/* Funci Consulta Aplicaci per Codi                      */
/* ------------------------------------------------------- */
/* USAGE : call PKG_APP.consulta_APP (<codi Aplicaci>);   */
/* ------------------------------------------------------- */
FUNCTION consulta_APP (CODI_APP IN VARCHAR2) RETURN BOOLEAN
IS  
  /* Crea el cursor de tipus APLICACION */
  CURSOR LLISTA_APP IS SELECT * FROM UOC.APLICACION WHERE APP_ID=CODI_APP;  

BEGIN  
   /* Obra el cursor */
   OPEN LLISTA_APP;  
   LOOP  
      /* Llegeix el cursor */
      FETCH LLISTA_APP INTO APP;  
      /* Si la llista est buida no ha trobat l'aplicaci */
      IF LLISTA_APP%ROWCOUNT=0 
         THEN 
             dbms_output.put_line('No s''ha trobat l''aplicaci.');
             RETURN FALSE;
         ELSE
             RETURN TRUE;
      END IF;   
      EXIT WHEN LLISTA_APP%NOTFOUND;  
   END LOOP;  
CLOSE LLISTA_APP;  

EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
   END;

END consulta_APP;

/* ------------------------------------------------------------ */
/* Funci Consulta Versi                                       */
/* ------------------------------------------------------------ */
/* USAGE : call PKG_APP.consulta_VERSIO (<codi App, Versio>);   */
/* ------------------------------------------------------------ */
FUNCTION consulta_VERSIO (in_CODI_APP IN VARCHAR2, in_VERSION IN VARCHAR2) RETURN BOOLEAN
IS  
  /* Crea el cursor de tipus APLICACION */
  CURSOR LLISTA_APP IS SELECT * FROM UOC.APLICACION WHERE (APP_ID=in_CODI_APP AND VERSION=in_VERSION);  

BEGIN  
   /* Obra el cursor */
   OPEN LLISTA_APP;  
   LOOP  
      /* Llegeix el cursor */
      FETCH LLISTA_APP INTO APP;  
      /* Si la llista est buida no ha trobat l'aplicaci */
      IF LLISTA_APP%ROWCOUNT=0 
         THEN dbms_output.put_line('No s''ha trobat la versi!');
             RETURN FALSE;
         ELSE
             RETURN TRUE;
      END IF;   
      EXIT WHEN LLISTA_APP%NOTFOUND;  
   END LOOP;  
CLOSE LLISTA_APP;  

EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
   END;

END consulta_VERSIO;

/* -------------------------------------------------------------- */
/* Procediment Alta Aplicaci                               */
/* -------------------------------------------------------------- */
/* USAGE : call PKG_APP.alta_APP (<dades aplicaci>);      */
/* -------------------------------------------------------------- */
PROCEDURE alta_APP (in_APP_ID IN VARCHAR2, in_VERSION IN VARCHAR2, in_SO_NOMBRE IN VARCHAR2, in_PAIS_ID IN CHAR, in_DESA_ID IN VARCHAR2, in_RES_VALOR IN VARCHAR2, in_APP_ACTIVA IN CHAR, in_APP_PRECIO IN NUMBER, in_APP_DESCRIPCION IN VARCHAR2, in_APP_PESO IN NUMBER, in_APP_VIDEO IN VARCHAR2, in_APP_FICHERO IN VARCHAR2, RSP OUT VARCHAR2)
IS
  in_APP_BINARIO BFILE;
  PARAM_ENTRADA VARCHAR2(500);

BEGIN
PARAM_ENTRADA := in_APP_ID + ', ' + in_VERSION + ', ' + in_SO_NOMBRE + ', ' + in_PAIS_ID + ', ' + in_DESA_ID + ', ' + in_RES_VALOR + ', ' + in_APP_ACTIVA + ', ' + in_APP_PRECIO + ', ' + in_APP_DESCRIPCION + ', ' + in_APP_PESO + ', ' + in_APP_VIDEO;
/* Comproba l'existncia del aplicaci */
IF NOT consulta_APP(in_APP_ID) 
THEN
  /* Afegeix el aplicaci a la taula APLICACION */
  in_APP_BINARIO := BFILENAME('DIR_FILES', in_APP_FICHERO);

  INSERT INTO UOC.APLICACION (APP_ID, VERSION, SO_NOMBRE, PAIS_ID, DESA_ID, RES_VALOR, APP_ACTIVA, APP_PRECIO, APP_DESCRIPCION, APP_PESO, APP_VIDEO, APP_BINARIO)
  VALUES (in_APP_ID, in_VERSION, in_SO_NOMBRE, in_PAIS_ID, in_DESA_ID, in_RES_VALOR, in_APP_ACTIVA, in_APP_PRECIO, in_APP_DESCRIPCION, in_APP_PESO, in_APP_VIDEO, in_APP_BINARIO);
  COMMIT WORK;
  IF consulta_APP(in_APP_ID) 
     THEN
         DBMS_OUTPUT.put_line('S''ha donat d''alta el registre segent: ');
         RSP := 'OK';
         mostra_APP;
     ELSE
         RSP := 'Error : No s''ha trobat l''aplicaci.';
         DBMS_OUTPUT.put_line('Error en el procs d''alta de l''aplicaci amb codi ' || APP.APP_ID || '.');
  END IF;
ELSE
    RSP := 'Error : L''aplicaci ja existeix.';
    DBMS_OUTPUT.put_line('Error: l''aplicaci amb codi ' || APP.APP_ID || ' ja existeix.');
END IF;

/* Compte el nmero de Aplicaci donats d'alta */
compta_APP;

/* Registre de log */
PKG_COMUN.reg_LOG ('alta_APP', PARAM_ENTRADA, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('alta_APP', PARAM_ENTRADA, RSP);
   END;

END alta_APP;
 
/* -------------------------------------------------------------- */
/* Procediment Baixa Aplicaci                              */
/* -------------------------------------------------------------- */
/* USAGE : call PKG_APP.baixa_APP (<codi aplicaci>);      */
/* -------------------------------------------------------------- */
PROCEDURE baixa_APP (CODI_APP IN VARCHAR2, RSP OUT VARCHAR2) 
IS

BEGIN  
/* Comproba l'existncia del aplicaci */
IF consulta_APP(CODI_APP) 
THEN
   /* Comproba l'existncia del aplicaci */
   DELETE FROM UOC.APLICACION WHERE (APP_ID = APP.APP_ID);
   COMMIT WORK;
   DBMS_OUTPUT.put_line('S''ha donat de baixa l''aplicaci amb codi ' || APP.APP_ID || '.');
   RSP := 'OK';
   /* Compte el nmero d'aplicacions donades d'alta */
   compta_APP;
END IF;

/* Registre de log */
PKG_COMUN.reg_LOG ('baixa_APP', CODI_APP, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('baixa_APP', CODI_APP, RSP);
   END;

END baixa_APP;

/* -------------------------------------------------------------- */
/* Procediment Baixa Versi                                       */
/* --------------------------------------------- ---------------- */
/* USAGE : call PKG_APP.baixa_VERSIO (<codi apli>,<versio>);      */
/* -------------------------------------------------------------- */
PROCEDURE baixa_VERSIO (in_CODI_APP IN VARCHAR2, in_VERSION IN VARCHAR2, RSP OUT VARCHAR2) 
IS

BEGIN  
/* Comproba l'existncia de la versi de l'aplicaci */
IF consulta_VERSIO (in_CODI_APP, in_VERSION) 
THEN
   DELETE FROM UOC.APLICACION WHERE (APP_ID = APP.APP_ID AND VERSION = APP.VERSION);
   COMMIT WORK;
   DBMS_OUTPUT.put_line('S''ha donat de baixa la versi ' || APP.VERSION || ' de l''aplicaci amb codi ' || APP.APP_ID || '.');
   RSP := 'OK';
END IF;

/* Registre de log */
PKG_COMUN.reg_LOG ('baixa_VERSIO', in_CODI_APP, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('baixa_VERSIO', in_CODI_APP, RSP);
   END;

END baixa_VERSIO;

/* --------------------------------------------------------- */
/* Procediment Modificaci aplicaci                         */
/* --------------------------------------------------------- */
/* USAGE : call PKG_APP.modif_APP (<dades aplicaci>);       */
/* --------------------------------------------------------- */
PROCEDURE modif_APP (in_APP_ID IN VARCHAR2, in_VERSION IN VARCHAR2, in_SO_NOMBRE IN VARCHAR2, in_PAIS_ID IN CHAR, in_DESA_ID IN VARCHAR2, in_RES_VALOR IN VARCHAR2, in_APP_ACTIVA IN CHAR, in_APP_PRECIO IN NUMBER, in_APP_DESCRIPCION IN VARCHAR2, in_APP_PESO IN NUMBER, in_APP_VIDEO IN VARCHAR2, in_APP_FICHERO IN VARCHAR2, RSP OUT VARCHAR2)
IS
  in_APP_BINARIO BFILE;
  PARAM_ENTRADA VARCHAR2(500);

BEGIN
PARAM_ENTRADA := in_APP_ID + ', ' + in_VERSION + ', ' + in_SO_NOMBRE + ', ' + in_PAIS_ID + ', ' + in_DESA_ID + ', ' + in_RES_VALOR + ', ' + in_APP_ACTIVA + ', ' + in_APP_PRECIO + ', ' + in_APP_DESCRIPCION + ', ' + in_APP_PESO + ', ' + in_APP_VIDEO;
/* Comproba l'existncia del aplicaci */
IF consulta_VERSIO(in_APP_ID, in_VERSION) 
THEN
  /* Modifica el aplicaci */
  in_APP_BINARIO := BFILENAME('DIR_FILES', in_APP_FICHERO);
  UPDATE UOC.APLICACION SET SO_NOMBRE=in_SO_NOMBRE, PAIS_ID=in_PAIS_ID, DESA_ID=in_DESA_ID, RES_VALOR=in_RES_VALOR, APP_ACTIVA=in_APP_ACTIVA, APP_PRECIO=in_APP_PRECIO, APP_DESCRIPCION=in_APP_DESCRIPCION, APP_PESO=in_APP_PESO, APP_VIDEO=in_APP_VIDEO, APP_BINARIO=in_APP_BINARIO;
  COMMIT WORK;
  /* Treu per pantalla les dades del aplicaci modificat */
  DBMS_OUTPUT.put_line('S''ha actualitzat el registre segent: ');
  mostra_APP;
  RSP := 'OK';
ELSE
    DBMS_OUTPUT.put_line('Error: L''aplicaci amb codi ' || in_APP_ID || ' no existeix.');
    RSP := 'Error: L''aplicaci amb codi ' + in_APP_ID + ' no existeix.';
END IF;

/* Registre de log */
PKG_COMUN.reg_LOG ('modif_APP', PARAM_ENTRADA, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('modif_APP', PARAM_ENTRADA, RSP);
   END;

END modif_APP;

END PKG_APP;
/
show errors
