-- +----------------------------------------------------------------------+
-- |                         Jordi Pascual Aran  (JPL)                    |
-- |                         DBA UOC                                      |
-- |----------------------------------------------------------------------|
-- | DATABASE : Oracle                                                    |
-- | FILE     : PKG_DESA.sql                                              |
-- | CLASS    : Install                                                   |
-- | PURPOSE  : Procediments ABM_DESARROLLADOR                            |
-- | DATE     : 26/11/2012                                                |
-- | VERSION  : Oracle 10g                                                |
-- |----------------------------------------------------------------------|
-- | USAGE:    sqlplus UOC/UOC @PKG_DESA.sql                              |
-- |----------------------------------------------------------------------|

set serveroutput on;

/* Capalera del paquet */
CREATE OR REPLACE PACKAGE PKG_DESA AS

FUNCTION consulta_DESA (CODI_DESA IN VARCHAR2) RETURN BOOLEAN;

PROCEDURE alta_DESA (in_DESA_ID IN VARCHAR2, in_DESA_EMPRESA IN VARCHAR2, in_PAIS_ID IN CHAR, in_DESA_REPRESENTANTE IN VARCHAR2, in_DIR_CALLE IN VARCHAR2, in_DIR_CP IN VARCHAR2, in_DIR_CIUDAD IN VARCHAR2, in_DIR_PROVINCIA IN VARCHAR2, in_DIR_PAIS IN CHAR, in_DESA_TELFPROF IN VARCHAR2, in_DESA_TELFMOVIL IN VARCHAR2, in_DESA_TELFPERSO IN VARCHAR2, in_DESA_FAX IN VARCHAR2, in_DESA_EMAIL IN VARCHAR2, in_DESA_WEB IN VARCHAR2, RSP OUT VARCHAR2);

PROCEDURE baixa_DESA (CODI_DESA IN VARCHAR2, RSP OUT VARCHAR2);

PROCEDURE modif_DESA (in_DESA_ID IN VARCHAR2, in_DESA_EMPRESA IN VARCHAR2, in_PAIS_ID IN CHAR, in_DESA_REPRESENTANTE IN VARCHAR2, in_DIR_CALLE IN VARCHAR2, in_DIR_CP IN VARCHAR2, in_DIR_CIUDAD IN VARCHAR2, in_DIR_PROVINCIA IN VARCHAR2, in_DIR_PAIS IN CHAR, in_DESA_TELFPROF IN VARCHAR2, in_DESA_TELFMOVIL IN VARCHAR2, in_DESA_TELFPERSO IN VARCHAR2, in_DESA_FAX IN VARCHAR2, in_DESA_EMAIL IN VARCHAR2, in_DESA_WEB IN VARCHAR2, RSP OUT VARCHAR2);

DESA UOC.DESARROLLADOR%ROWTYPE;

END PKG_DESA;
/

/* Cos del paquet */
CREATE OR REPLACE PACKAGE BODY PKG_DESA AS


/* Procediment que treu per pantalla les dades d'un desenvolupador */
PROCEDURE mostra_DESA
IS
BEGIN
  DBMS_OUTPUT.put_line('Codi: ' || DESA.DESA_ID);
  DBMS_OUTPUT.put_line('Empresa: ' || DESA.DESA_EMPRESA);
  DBMS_OUTPUT.put_line('Pas de registre: ' || DESA.PAIS_ID);
  DBMS_OUTPUT.put_line('Representant: ' || DESA.DESA_REPRESENTANTE);
  DBMS_OUTPUT.put_line('Adrea: ' || DESA.DIR_CALLE || ',' || DESA.DIR_CP);
  DBMS_OUTPUT.put_line('        ' || DESA.DIR_CIUDAD || ',' || DESA.DIR_PROVINCIA || ',' || DESA.DIR_PAIS);
  DBMS_OUTPUT.put_line('Telfons: ' || DESA.DESA_TELFPROF || ',' || DESA.DESA_TELFMOVIL || ',' || DESA.DESA_TELFPERSO);
  DBMS_OUTPUT.put_line('Fax: ' || DESA.DESA_FAX);
  DBMS_OUTPUT.put_line('Email: ' || DESA.DESA_EMAIL);
  DBMS_OUTPUT.put_line('Web: ' || DESA.DESA_WEB);
END ;

/* Procediment compta el nmero total de desenvolupadors donats d'alta */
PROCEDURE compta_DESA
IS
  num number;
BEGIN
  /* Compte el nmero de desenvolupadors donats d'alta */
  SELECT COUNT(*) INTO num FROM UOC.DESARROLLADOR;
  /* Mostra el resultat per pantalla */
  DBMS_OUTPUT.put_line('Actualment hi han ' || num || ' desenvolupadors donats d''alta.');
END ;

/* -------------------------------------------------------------- */
/* Funci Consulta Desenvolupador per Codi                        */
/* -------------------------------------------------------------- */
/* USAGE : call PKG_DESA.consulta_DESA (<codi desenvolupador>);   */
/* -------------------------------------------------------------- */
FUNCTION consulta_DESA (CODI_DESA IN VARCHAR2) RETURN BOOLEAN
IS  
  /* Crea el cursor de tipus DESARROLLADOR */
  CURSOR LLISTA_DESA IS SELECT * FROM UOC.DESARROLLADOR WHERE DESA_ID=CODI_DESA;  

BEGIN  
   /* Obra el cursor */
   OPEN LLISTA_DESA;  
   LOOP  
      /* Llegeix el cursor */
      FETCH LLISTA_DESA INTO DESA;  
      /* Si la llista est buida no ha trobat el codi de desenvolupador */
      IF LLISTA_DESA%ROWCOUNT=0 
         THEN dbms_output.put_line('No s''ha trobat el desenvolupador.');
             RETURN FALSE;
         ELSE
             RETURN TRUE;
      END IF;   
      EXIT WHEN LLISTA_DESA%NOTFOUND;  
   END LOOP;  
CLOSE LLISTA_DESA;  

EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
   END;

END consulta_DESA;

/* -------------------------------------------------------------- */
/* Procediment Alta Desenvolupadors                               */
/* -------------------------------------------------------------- */
/* USAGE : call PKG_DESA.alta_DESA (<dades desenvolupador>);      */
/* -------------------------------------------------------------- */
--PROCEDURE alta_DESA (in_DESA IN UOC.DESARROLLADOR%ROWTYPE)
PROCEDURE alta_DESA (in_DESA_ID IN VARCHAR2, in_DESA_EMPRESA IN VARCHAR2, in_PAIS_ID IN CHAR, in_DESA_REPRESENTANTE IN VARCHAR2, in_DIR_CALLE IN VARCHAR2, in_DIR_CP IN VARCHAR2, in_DIR_CIUDAD IN VARCHAR2, in_DIR_PROVINCIA IN VARCHAR2, in_DIR_PAIS IN CHAR, in_DESA_TELFPROF IN VARCHAR2, in_DESA_TELFMOVIL IN VARCHAR2, in_DESA_TELFPERSO IN VARCHAR2, in_DESA_FAX IN VARCHAR2, in_DESA_EMAIL IN VARCHAR2, in_DESA_WEB IN VARCHAR2, RSP OUT VARCHAR2)
IS
PARAM_ENTRADA VARCHAR2(500);

BEGIN
PARAM_ENTRADA := in_DESA_ID + ', ' + in_DESA_EMPRESA + ', ' + in_PAIS_ID + ', ' + in_DESA_REPRESENTANTE + ', ' + in_DIR_CALLE + ', ' + in_DIR_CP + ', ' + in_DIR_CIUDAD + ', ' + in_DIR_PROVINCIA + ', ' + in_DIR_PAIS + ', ' + in_DESA_TELFPROF + ', ' + in_DESA_TELFMOVIL + ', ' + in_DESA_TELFPERSO + ', ' + in_DESA_FAX + ', ' + in_DESA_EMAIL + ', ' + in_DESA_WEB;
/* Comproba l'existncia del desenvolupador */
IF NOT consulta_DESA(in_DESA_ID) 
THEN
  /* Afegeix el desenvolupador a la taula DESARROLLADOR */
  INSERT INTO UOC.DESARROLLADOR (DESA_ID, DESA_EMPRESA, PAIS_ID, DESA_REPRESENTANTE, DIR_CALLE, DIR_CP, DIR_CIUDAD, DIR_PROVINCIA, DIR_PAIS, DESA_TELFPROF, DESA_TELFMOVIL, DESA_TELFPERSO, DESA_FAX, DESA_EMAIL, DESA_WEB)
  VALUES (in_DESA_ID, in_DESA_EMPRESA, in_PAIS_ID, in_DESA_REPRESENTANTE, in_DIR_CALLE, in_DIR_CP, in_DIR_CIUDAD, in_DIR_PROVINCIA, in_DIR_PAIS, in_DESA_TELFPROF, in_DESA_TELFMOVIL, in_DESA_TELFPERSO, in_DESA_FAX, in_DESA_EMAIL, in_DESA_WEB);
  COMMIT WORK;
  IF consulta_DESA(in_DESA_ID) 
     THEN
         DBMS_OUTPUT.put_line('S''ha donat d''alta el registre segent: ');
         mostra_DESA;
         RSP := 'OK';
     ELSE
         DBMS_OUTPUT.put_line('Error en el procs d''alta del desenvolupador amb codi ' || DESA.DESA_ID || '.');
         RSP := 'Error en el procs d''alta del desenvolupador amb codi ' + DESA.DESA_ID;
  END IF;
ELSE
    DBMS_OUTPUT.put_line('Error: el desenvolupador amb codi ' || DESA.DESA_ID || ' ja existeix.');
    RSP := 'Error: el desenvolupador amb codi ' + DESA.DESA_ID + ' ja existeix.';
END IF;

/* Compte el nmero de desenvolupadors donats d'alta */
compta_DESA;

/* Registre de log */
PKG_COMUN.reg_LOG ('alta_DESA', PARAM_ENTRADA, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('alta_DESA', PARAM_ENTRADA, RSP);
   END;

END alta_DESA;
 
/* -------------------------------------------------------------- */
/* Procediment Baixa Desenvolupadors                              */
/* -------------------------------------------------------------- */
/* USAGE : call PKG_DESA.baixa_DESA (<codi desenvolupador>);      */
/* -------------------------------------------------------------- */
PROCEDURE baixa_DESA (CODI_DESA IN VARCHAR2, RSP OUT VARCHAR2) 
IS

BEGIN  
/* Comproba l'existncia del desenvolupador */
IF consulta_DESA(CODI_DESA) 
THEN
   /* Comproba l'existncia del desenvolupador */
   DELETE FROM UOC.DESARROLLADOR WHERE (DESA_ID = DESA.DESA_ID);
   COMMIT WORK;
   DBMS_OUTPUT.put_line('S''ha donat de baixa el desenvolupador amb codi ' || DESA.DESA_ID || '.');
   RSP := 'OK';
ELSE
   RSP := 'Error: No s''ha trobat el desenvolupador.';
END IF;
/* Compte el nmero de desenvolupadors donats d'alta */
compta_DESA;

/* Registre de log */
PKG_COMUN.reg_LOG ('baixa_DESA', CODI_DESA, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('baixa_DESA', CODI_DESA, RSP);
   END;

END baixa_DESA;

/* -------------------------------------------------------------- */
/* Procediment Modificaci Desenvolupador                         */
/* -------------------------------------------------------------- */
/* USAGE : call PKG_DESA.modif_DESA (<dades desenvolupador>);     */
/* -------------------------------------------------------------- */
PROCEDURE modif_DESA (in_DESA_ID IN VARCHAR2, in_DESA_EMPRESA IN VARCHAR2, in_PAIS_ID IN CHAR, in_DESA_REPRESENTANTE IN VARCHAR2, in_DIR_CALLE IN VARCHAR2, in_DIR_CP IN VARCHAR2, in_DIR_CIUDAD IN VARCHAR2, in_DIR_PROVINCIA IN VARCHAR2, in_DIR_PAIS IN CHAR, in_DESA_TELFPROF IN VARCHAR2, in_DESA_TELFMOVIL IN VARCHAR2, in_DESA_TELFPERSO IN VARCHAR2, in_DESA_FAX IN VARCHAR2, in_DESA_EMAIL IN VARCHAR2, in_DESA_WEB IN VARCHAR2, RSP OUT VARCHAR2)
IS
PARAM_ENTRADA VARCHAR2(500);

BEGIN
PARAM_ENTRADA := in_DESA_ID + ', ' + in_DESA_EMPRESA + ', ' + in_PAIS_ID + ', ' + in_DESA_REPRESENTANTE + ', ' + in_DIR_CALLE + ', ' + in_DIR_CP + ', ' + in_DIR_CIUDAD + ', ' + in_DIR_PROVINCIA + ', ' + in_DIR_PAIS + ', ' + in_DESA_TELFPROF + ', ' + in_DESA_TELFMOVIL + ', ' + in_DESA_TELFPERSO + ', ' + in_DESA_FAX + ', ' + in_DESA_EMAIL + ', ' + in_DESA_WEB;
/* Comproba l'existncia del desenvolupador */
IF consulta_DESA(in_DESA_ID) 
THEN
  /* Modifica el desenvolupador */
  UPDATE UOC.DESARROLLADOR SET DESA_EMPRESA=in_DESA_EMPRESA, PAIS_ID=in_PAIS_ID, DESA_REPRESENTANTE=in_DESA_REPRESENTANTE, DIR_CALLE=in_DIR_CALLE, DIR_CP=in_DIR_CP, DIR_CIUDAD=in_DIR_CIUDAD, DIR_PROVINCIA=in_DIR_PROVINCIA, DIR_PAIS=in_DIR_PAIS, DESA_TELFPROF=in_DESA_TELFPROF, DESA_TELFMOVIL=in_DESA_TELFMOVIL, DESA_TELFPERSO=in_DESA_TELFPERSO, DESA_FAX=in_DESA_FAX, DESA_EMAIL=in_DESA_EMAIL, DESA_WEB=in_DESA_WEB WHERE DESA_ID=in_DESA_ID;
  COMMIT WORK;
  /* Treu per pantalla les dades del desenvolupador modificat */
  DBMS_OUTPUT.put_line('S''ha actualitzat el registre segent: ');
  mostra_DESA;
  RSP := 'OK';
ELSE
    DBMS_OUTPUT.put_line('Error: el desenvolupador amb codi ' || DESA.DESA_ID || ' no existeix.');
    RSP := 'Error: el desenvolupador amb codi ' + DESA.DESA_ID + ' no existeix.';
END IF;

/* Registre de log */
PKG_COMUN.reg_LOG ('consulta_DESA', PARAM_ENTRADA, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('consulta_DESA', PARAM_ENTRADA, RSP);
   END;

END modif_DESA;

END PKG_DESA;
/
show errors
