-- +----------------------------------------------------------------------+
-- |                         Jordi Pascual Aran  (JPL)                    |
-- |                         DBA UOC                                      |
-- |----------------------------------------------------------------------|
-- | DATABASE : Oracle                                                    |
-- | FILE     : PKG_DESCARGA.sql                                          |
-- | CLASS    : Install                                                   |
-- | PURPOSE  : Procediments DESCARGA                                     |
-- | DATE     : 13/12/2012                                                |
-- | VERSION  : Oracle 10g                                                |
-- |----------------------------------------------------------------------|
-- | USAGE:    sqlplus UOC/UOC @PKG_DESCARGA.sql                          |
-- |----------------------------------------------------------------------|

set serveroutput on;

/* Capalera del paquet */
CREATE OR REPLACE PACKAGE PKG_DESCARGA AS

PROCEDURE alta_DESCARGA (in_APP_ID IN VARCHAR2, in_VERSION IN VARCHAR2, in_IMEI IN VARCHAR2, in_PAGO_NOMBRE IN VARCHAR2, RSP OUT VARCHAR2);

PROCEDURE consulta_DESCARGA (in_IMEI IN VARCHAR2, RSP OUT VARCHAR2);

PROCEDURE consulta_DESCARGA (in_APP_ID IN VARCHAR2, in_VERSION IN VARCHAR2, RSP OUT VARCHAR2);

PROCEDURE consulta_DESCARGA (in_APP_ID IN VARCHAR2, in_VERSION IN VARCHAR2, in_IMEI IN VARCHAR2, in_PAGO_NOMBRE IN VARCHAR2);

APP UOC.APLICACION%ROWTYPE;  
DISPO UOC.DISPOSITIVO%ROWTYPE;
USER UOC.USUARIO%ROWTYPE;  
DESCARGA UOC.DESCARGA%ROWTYPE;  

END PKG_DESCARGA;
/

/* Cos del paquet */
CREATE OR REPLACE PACKAGE BODY PKG_DESCARGA AS

/* Procediment que treu per pantalla les dades d'un usuari */
PROCEDURE mostra_DESCARGA
IS
BEGIN
  DBMS_OUTPUT.put_line('Usuari: ' || USER.USR_MOVIL);
  DBMS_OUTPUT.put_line('Hora descrrega: ' || DESCARGA.DESCARGA_FECHA);
  DBMS_OUTPUT.put_line('Mode de pagament: ' || DESCARGA.PAGO_NOMBRE);
  DBMS_OUTPUT.put_line('Aplicaci descarregada: Aplicaci ' || DESCARGA.APP_ID || '; Versi ' || DESCARGA.VERSION);
  DBMS_OUTPUT.put_line('Preu: ' || DESCARGA.DESCARGA_PRECIO);
  DBMS_OUTPUT.put_line('IMEI Dispositiu: ' || DESCARGA.IMEI);
  DBMS_OUTPUT.put_line('Pas de registre: ' || DESCARGA.DESCARGA_PAIS);
  DBMS_OUTPUT.put_line('Operador: ' || DESCARGA.DESCARGA_OPERADOR);
END ;

/* Procediment compta el nmero total de descrregues */
PROCEDURE compta_DESCARGA
IS
  num number;
BEGIN
  /* Compte el nmero de descrregues */
  SELECT COUNT(*) INTO num FROM UOC.DESCARGA;
  /* Mostra el resultat per pantalla */
  DBMS_OUTPUT.put_line('S''han realitzat ' || num || ' descrregues.');
END ;

/* --------------------------------------------------- */
/* Procediment Consulta Descrregues                   */
/* --------------------------------------------------- */
/* USAGE : call PKG_DESCARGA.consulta_DESCARGA (IMEI); */
/* ----------------------------------------------------*/
PROCEDURE consulta_DESCARGA (in_IMEI IN VARCHAR2, RSP OUT VARCHAR2)
IS  
  CURSOR LLISTA_DESCARGA IS SELECT * FROM UOC.DESCARGA WHERE (IMEI=in_IMEI);  

BEGIN  
RSP := 'OK';
   /* Obra el cursor */
  OPEN LLISTA_DESCARGA;
  LOOP  
      /* Llegeix el cursor aplicaci*/
      FETCH LLISTA_DESCARGA INTO DESCARGA;  
      /* Si la llista est buida no ha trobat l'aplicaci */
      IF LLISTA_DESCARGA%ROWCOUNT=0 
         THEN 
             dbms_output.put_line('No s''ha trobat cap descrrega.');
             RSP := 'Error: No s''ha trobat cap descrrega.';
         ELSE
             mostra_DESCARGA;
      END IF;   
      EXIT WHEN LLISTA_DESCARGA%NOTFOUND;  
   END LOOP;  
CLOSE LLISTA_DESCARGA;  

/* Registre de log */
PKG_COMUN.reg_LOG ('consulta_DESCARGA', in_IMEI, RSP);

EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        PKG_COMUN.reg_LOG ('consulta_DESCARGA', in_IMEI, RSP);
   END;

END consulta_DESCARGA;

/* --------------------------------------------------- */
/* Procediment Consulta Descrregues                   */
/* --------------------------------------------------- */
/* USAGE : call PKG_DESCARGA.consulta_DESCARGA (Apli); */
/* ----------------------------------------------------*/

PROCEDURE consulta_DESCARGA (in_APP_ID IN VARCHAR2, in_VERSION IN VARCHAR2, RSP OUT VARCHAR2)
IS  
  CURSOR LLISTA_DESCARGA IS SELECT * FROM UOC.DESCARGA WHERE (APP_ID=in_APP_ID and VERSION=in_VERSION);  
  PARAM_ENTRADA VARCHAR2(500);

BEGIN  
PARAM_ENTRADA := in_APP_ID + ', ' + in_VERSION;
RSP := 'OK';
   /* Obra el cursor */
  OPEN LLISTA_DESCARGA;
  LOOP  
      /* Llegeix el cursor aplicaci*/
      FETCH LLISTA_DESCARGA INTO DESCARGA;  
      /* Si la llista est buida no ha trobat l'aplicaci */
      IF LLISTA_DESCARGA%ROWCOUNT=0 
         THEN 
             dbms_output.put_line('No s''ha trobat cap descrrega.');
             RSP := 'Error: No s''ha trobat cap descrrega.';
         ELSE
             mostra_DESCARGA;
      END IF;   
      EXIT WHEN LLISTA_DESCARGA%NOTFOUND;  
   END LOOP;  
CLOSE LLISTA_DESCARGA;  

/* Registre de log */
PKG_COMUN.reg_LOG ('consulta_DESCARGA', PARAM_ENTRADA, RSP);

EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        PKG_COMUN.reg_LOG ('consulta_DESCARGA', PARAM_ENTRADA, RSP);
   END;

END consulta_DESCARGA;

/* --------------------------------------------------- */
/* Procediment Consulta Descrregues                   */
/* --------------------------------------------------- */
/* USAGE: call PKG_DESCARGA.consulta_DESCARGA (dades); */
/* ----------------------------------------------------*/

PROCEDURE consulta_DESCARGA (in_APP_ID IN VARCHAR2, in_VERSION IN VARCHAR2, in_IMEI IN VARCHAR2, in_PAGO_NOMBRE IN VARCHAR2)
IS  
  CURSOR LLISTA_DESCARGA IS SELECT * FROM UOC.DESCARGA WHERE (APP_ID=in_APP_ID and VERSION=in_VERSION and IMEI=in_IMEI and PAGO_NOMBRE=in_PAGO_NOMBRE);  

BEGIN  
   /* Obra el cursor */
  OPEN LLISTA_DESCARGA;
  LOOP  
      /* Llegeix el cursor aplicaci*/
      FETCH LLISTA_DESCARGA INTO DESCARGA;  
      /* Si la llista est buida no ha trobat l'aplicaci */
      IF LLISTA_DESCARGA%ROWCOUNT=0 
         THEN 
             dbms_output.put_line('No s''ha trobat cap descrrega.');
         ELSE
             mostra_DESCARGA;
      END IF;   
      EXIT WHEN LLISTA_DESCARGA%NOTFOUND;  
   END LOOP;  
CLOSE LLISTA_DESCARGA;  

EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
   END;

END consulta_DESCARGA;

/* ------------------------------------------------------ */
/* Procediment Alta Descrrega                            */
/* ------------------------------------------------------ */
/* USAGE : call PKG_DESCARGA.alta_DESCARGA (<dades>);     */
/* ------------------------------------------------------ */
PROCEDURE alta_DESCARGA (in_APP_ID IN VARCHAR2, in_VERSION IN VARCHAR2, in_IMEI IN VARCHAR2, in_PAGO_NOMBRE IN VARCHAR2, RSP OUT VARCHAR2)
IS
  PARAM_ENTRADA VARCHAR2(500);
  CURSOR LLISTA_APP IS SELECT * FROM UOC.APLICACION WHERE (APP_ID=in_APP_ID AND VERSION=in_VERSION);  
  CURSOR LLISTA_DISPO IS SELECT * FROM UOC.DISPOSITIVO WHERE IMEI=in_IMEI;  
  CURSOR LLISTA_USER IS SELECT * FROM UOC.USUARIO WHERE USR_MOVIL=DISPO.USR_MOVIL;  

BEGIN
PARAM_ENTRADA := in_APP_ID + ', ' + in_VERSION + ', ' + in_IMEI + ', ' + in_PAGO_NOMBRE;
/* Comproba l'existncia de l'aplicaci */
--   OPEN LLISTA_APP;  
--      /* Llegeix el cursor aplicaci*/
--      FETCH LLISTA_APP INTO APP;  
--      /* Si la llista est buida => l'aplicaci no existeix */
--      IF LLISTA_APP%ROWCOUNT=0 

         IF PKG_APP.consulta_VERSIO (in_APP_ID, in_VERSION)
         THEN 
             dbms_output.put_line('No s''ha trobat cap aplicaci.');
             RSP := 'Error: No s''ha trobat cap aplicaci.';
         ELSE
             /* Llegeix el cursor dispositiu*/
            OPEN LLISTA_DISPO;
            FETCH LLISTA_DISPO INTO DISPO;  
            /* Si la llista est buida => el dispositiu no existeix */
            IF LLISTA_DISPO%ROWCOUNT=0 
               THEN 
                   dbms_output.put_line('No s''ha trobat cap dispositiu.');
                   RSP := 'Error: No s''ha trobat el dispositiu.';
               ELSE
                     /* Llegeix el cursor usuari*/
                     OPEN LLISTA_USER;
                     FETCH LLISTA_USER INTO USER;                     
                     /* Si la llista est buida => l'usuari no existeix */
                     IF LLISTA_USER%ROWCOUNT=0 
                         THEN 
                             dbms_output.put_line('No s''ha trobat cap usuari.');
                             RSP := 'Error: No s''ha trobat cap usuari.';
                         ELSE
                            /* Enregistra la descrrega */
                            INSERT INTO UOC.DESCARGA (APP_ID, VERSION, DESCARGA_PRECIO, IMEI, DESCARGA_OPERADOR, DESCARGA_PAIS, PAGO_NOMBRE)
                            VALUES (APP.APP_ID, APP.VERSION, APP.APP_PRECIO, DISPO.IMEI, USER.OPERA_NOMBRE, USER.PAIS_ID, in_PAGO_NOMBRE);
                            COMMIT WORK;
                            DBMS_OUTPUT.put_line('S''ha enregistrat la descrrega: ');
--                            consulta_DESCARGA (in_APP_ID, in_VERSION, in_IMEI, in_PAGO_NOMBRE);
                            RSP := 'OK';
                            DBMS_OUTPUT.put_line('IMEI: ' || DISPO.IMEI);
                            DBMS_OUTPUT.put_line('Operador: ' || USER.OPERA_NOMBRE);
                            DBMS_OUTPUT.put_line('Pais: ' || USER.PAIS_ID);
                            DBMS_OUTPUT.put_line('Pago: ' || in_PAGO_NOMBRE);
                     END IF;
              END IF;
      END IF;
   
CLOSE LLISTA_USER;  
CLOSE LLISTA_DISPO;  
--CLOSE LLISTA_APP;  

/* Compte el nmero de usuaris donats d'alta */
compta_DESCARGA;

/* Registre de log */
PKG_COMUN.reg_LOG ('alta_DESCARGA', PARAM_ENTRADA, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('alta_DESCARGA', PARAM_ENTRADA, RSP);
   END;

END alta_DESCARGA;

END PKG_DESCARGA;
/
show errors
