-- +----------------------------------------------------------------------+
-- |                         Jordi Pascual Aran  (JPL)                    |
-- |                         DBA UOC                                      |
-- |----------------------------------------------------------------------|
-- | DATABASE : Oracle                                                    |
-- | FILE     : PKG_USER.sql                                              |
-- | CLASS    : Install                                                   |
-- | PURPOSE  : Procediments ABM_USUARIO                                  |
-- | DATE     : 28/11/2012                                                |
-- | VERSION  : Oracle 10g                                                |
-- |----------------------------------------------------------------------|
-- | USAGE:    sqlplus UOC/UOC @PKG_USER.sql                              |
-- |----------------------------------------------------------------------|

set serveroutput on;

/* Capalera del paquet */
CREATE OR REPLACE PACKAGE PKG_USER AS

FUNCTION consulta_USER (in_MOVIL_USER IN VARCHAR2) RETURN BOOLEAN;

PROCEDURE alta_USER (in_USR_MOVIL IN VARCHAR2, in_OPERA_NOMBRE IN VARCHAR2, in_PAIS_ID IN CHAR, in_USR_EMAIL IN VARCHAR2, in_IMEI IN VARCHAR2, in_SO_NOMBRE IN VARCHAR2, in_RES_VALOR IN VARCHAR2, in_MODELO IN VARCHAR2, RSP OUT VARCHAR2);

PROCEDURE alta_DISPO (in_USR_MOVIL IN VARCHAR2, in_IMEI IN VARCHAR2, in_PAIS_ID IN CHAR, in_SO_NOMBRE IN VARCHAR2, in_RES_VALOR IN VARCHAR2, in_MODELO IN VARCHAR2, RSP OUT VARCHAR2);

PROCEDURE baixa_USER (in_USR_MOVIL IN VARCHAR2, RSP OUT VARCHAR2); 

PROCEDURE modif_USER (in_USR_MOVIL IN VARCHAR2, in_OPERA_NOMBRE IN VARCHAR2, in_PAIS_ID IN CHAR, in_USR_EMAIL IN VARCHAR2, RSP OUT VARCHAR2);

PROCEDURE modif_DISPO (in_USR_MOVIL IN VARCHAR2, in_IMEI IN VARCHAR2, in_SO_NOMBRE IN VARCHAR2, in_RES_VALOR IN CHAR, in_MODELO IN VARCHAR2, RSP OUT VARCHAR2);

PROCEDURE baixa_DISPO (in_USR_MOVIL IN VARCHAR2, in_IMEI IN VARCHAR2, RSP OUT VARCHAR2); 

USER UOC.USUARIO%ROWTYPE;
DISPO UOC.DISPOSITIVO%ROWTYPE;

END PKG_USER;
/

/* Cos del paquet */
CREATE OR REPLACE PACKAGE BODY PKG_USER AS

/* Procediment que treu per pantalla les dades d'un usuari */
PROCEDURE mostra_USER
IS
BEGIN
  DBMS_OUTPUT.put_line('Telfon: ' || USER.USR_MOVIL);
  DBMS_OUTPUT.put_line('Operador: ' || USER.OPERA_NOMBRE);
  DBMS_OUTPUT.put_line('Pas de registre: ' || USER.PAIS_ID);
  DBMS_OUTPUT.put_line('Email: ' || USER.USR_EMAIL);
END ;

/* Procediment que treu per pantalla les dades d'un dispositiu */
PROCEDURE mostra_DISPO
IS
BEGIN
  DBMS_OUTPUT.put_line('IMEI: ' || DISPO.IMEI);
  DBMS_OUTPUT.put_line('Sistema Operatiu: ' || DISPO.SO_NOMBRE);
  DBMS_OUTPUT.put_line('Resoluci: ' || DISPO.RES_VALOR);
  DBMS_OUTPUT.put_line('Model: ' || DISPO.MODELO);
END ;

/* Procediment compta el nmero total de usuaris donats d'alta */
PROCEDURE compta_USER
IS
  num number;
BEGIN
  /* Compte el nmero de usuaris donats d'alta */
  SELECT COUNT(*) INTO num FROM UOC.USUARIO;
  /* Mostra el resultat per pantalla */
  DBMS_OUTPUT.put_line('Actualment hi han ' || num || ' usuaris donats d''alta.');
END ;

/* ------------------------------------------------------ */
/* Funci Consulta Usuari per Nmero Telfon              */
/* ------------------------------------------------------ */
/* USAGE : call PKG_USER.consulta_USER (<num.tel>);       */
/* ------------------------------------------------------ */
FUNCTION consulta_USER (in_MOVIL_USER IN VARCHAR2) RETURN BOOLEAN
IS  
  /* Crea el cursor de tipus USUARI */
  CURSOR LLISTA_USER IS SELECT * FROM UOC.USUARIO WHERE USR_MOVIL=in_MOVIL_USER;  

BEGIN  
   /* Obra el cursor */
   OPEN LLISTA_USER;  
   LOOP  
      /* Llegeix el cursor */
      FETCH LLISTA_USER INTO USER;  
      /* Si la llista est buida no ha trobat l'usuari */
      IF LLISTA_USER%ROWCOUNT=0 
         THEN dbms_output.put_line('No s''ha trobat cap usuari!');
             RETURN FALSE;
         ELSE
             RETURN TRUE;
      END IF;   
      EXIT WHEN LLISTA_USER%NOTFOUND;  
   END LOOP;  
CLOSE LLISTA_USER;  

EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
   END;

END consulta_USER;

/* Procediment compta el nmero de dispositius per usuari */
PROCEDURE compta_DISPO (in_MOVIL_USER IN VARCHAR2)
IS
  num number;
BEGIN
  /* Recupera les dades de l'usuari */
  IF consulta_USER(in_MOVIL_USER)
     THEN
         /* Compte el nmero de dispositius per l'usuari especificat */
         SELECT COUNT(*) INTO num FROM UOC.DISPOSITIVO WHERE USR_MOVIL=USER.USR_MOVIL;
         /* Mostra el resultat per pantalla */
         DBMS_OUTPUT.put_line('L''usuari ' || USER.USR_MOVIL || ' t associats ' || num || ' dispositius.');
     ELSE
         DBMS_OUTPUT.put_line('No s''ha trobat l''usuari amb telfon ' || USER.USR_MOVIL || '.');
  END IF;
END ;

/* ------------------------------------------------------ */
/* Funci Consulta Dispositiu per codi IMEI               */
/* ------------------------------------------------------ */
/* USAGE : call PKG_USER.consulta_DISPO (in_IMEI);        */
/* ------------------------------------------------------ */
FUNCTION consulta_DISPO (in_IMEI IN VARCHAR2) RETURN BOOLEAN
IS  
  /* Crea el cursor de tipus DISPOSITIU */
  CURSOR LLISTA_DISPO IS SELECT * FROM UOC.DISPOSITIVO WHERE IMEI=in_IMEI;  

BEGIN  
   /* Obra el cursor */
   OPEN LLISTA_DISPO;  
   LOOP  
      /* Llegeix el cursor */
      FETCH LLISTA_DISPO INTO DISPO;  
      /* Si la llista est buida no ha trobat el dispositiu */
      IF LLISTA_DISPO%ROWCOUNT=0 
         THEN 
             dbms_output.put_line('No s''ha trobat cap dispositiu!');
             RETURN FALSE;
         ELSE
             RETURN TRUE;
      END IF;   
      EXIT WHEN LLISTA_DISPO%NOTFOUND;  
   END LOOP;  
CLOSE LLISTA_DISPO;  

EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
   END;

END consulta_DISPO;

/* ------------------------------------------------------ */
/* Procedure Consulta Dispositiu per Telfon Usuari          */
/* ------------------------------------------------------ */
/* USAGE : call PKG_USER.consulta_DISPO (in_USR_MOVIL);   */
/* ------------------------------------------------------ */
PROCEDURE consulta_DISPO (in_USR_MOVIL IN VARCHAR2, RSP OUT VARCHAR2)
IS  
  /* Crea el cursor de tipus DISPOSITIU */
  CURSOR LLISTA_DISPO IS SELECT * FROM UOC.DISPOSITIVO WHERE USR_MOVIL=in_USR_MOVIL;  

BEGIN  
   /* Obra el cursor */
   OPEN LLISTA_DISPO;  
   LOOP  
      /* Llegeix el cursor */
      FETCH LLISTA_DISPO INTO DISPO;  
      /* Si la llista est buida no ha trobat el dispositiu */
      IF LLISTA_DISPO%ROWCOUNT=0 
         THEN 
             dbms_output.put_line('No s''ha trobat cap dispositiu.');
             RSP := 'No s''ha trobat cap dispositiu.';
         ELSE
             mostra_DISPO;
             RSP := 'OK';
      END IF;   
      EXIT WHEN LLISTA_DISPO%NOTFOUND;  
   END LOOP;  
CLOSE LLISTA_DISPO;  

/* Registre de log */
PKG_COMUN.reg_LOG ('consulta_DISPO', in_USR_MOVIL, RSP);

EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('consulta_DISPO', in_USR_MOVIL, RSP);
   END;

END consulta_DISPO;

/* ------------------------------------------------------ */
/* Procediment Alta Usuari                                */
/* ------------------------------------------------------ */
/* USAGE : call PKG_USER.alta_USER (<dades usuari>);      */
/* ------------------------------------------------------ */
PROCEDURE alta_USER (in_USR_MOVIL IN VARCHAR2, in_OPERA_NOMBRE IN VARCHAR2, in_PAIS_ID IN CHAR, in_USR_EMAIL IN VARCHAR2, in_IMEI IN VARCHAR2, in_SO_NOMBRE IN VARCHAR2, in_RES_VALOR IN VARCHAR2, in_MODELO IN VARCHAR2, RSP OUT VARCHAR2)
IS
  PARAM_ENTRADA VARCHAR2(500);

BEGIN
PARAM_ENTRADA := in_USR_MOVIL + ', ' + in_OPERA_NOMBRE + ', ' + in_PAIS_ID + ', ' + in_USR_EMAIL;
/* Comproba l'existncia del usuari */
IF NOT consulta_USER(in_USR_MOVIL) 
THEN
  /* Afegeix l'usuari a la taula USUARIO */
  INSERT INTO UOC.USUARIO (USR_MOVIL, OPERA_NOMBRE, PAIS_ID, USR_EMAIL)
  VALUES (in_USR_MOVIL, in_OPERA_NOMBRE, in_PAIS_ID, in_USR_EMAIL);
  /* Afegeix el dispositiu a la taula DISPOSITIVO */
  INSERT INTO UOC.DISPOSITIVO (IMEI, USR_MOVIL, SO_NOMBRE, RES_VALOR, MODELO)
  VALUES (in_IMEI, in_USR_MOVIL, in_SO_NOMBRE, in_RES_VALOR, in_MODELO);
  COMMIT WORK;
  DBMS_OUTPUT.put_line('S''ha donat d''alta l''usuari segent: ');
  mostra_USER;
  RSP := 'OK';
ELSE
    DBMS_OUTPUT.put_line('Error: No es pot donar d''alta l''usuari.');
    RSP := 'Error: No es pot donar d''alta l''usuari.';
END IF;

/* Compte el nmero de usuaris donats d'alta */
compta_USER;

/* Registre de log */
PKG_COMUN.reg_LOG ('alta_USER', PARAM_ENTRADA, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('alta_USER', PARAM_ENTRADA, RSP);
   END;

END alta_USER;
 
/* ------------------------------------------------------ */
/* Procediment Alta Dispositiu                            */
/* ------------------------------------------------------ */
/* USAGE : call PKG_USER.alta_DISPO (<dades dispositiu>); */
/* ------------------------------------------------------ */
PROCEDURE alta_DISPO (in_USR_MOVIL IN VARCHAR2, in_IMEI IN VARCHAR2, in_PAIS_ID IN CHAR, in_SO_NOMBRE IN VARCHAR2, in_RES_VALOR IN VARCHAR2, in_MODELO IN VARCHAR2, RSP OUT VARCHAR2)
IS
  PARAM_ENTRADA VARCHAR2(500);

BEGIN
PARAM_ENTRADA := in_IMEI + ', ' + in_USR_MOVIL + ', ' + in_SO_NOMBRE + ', ' + in_RES_VALOR + ', ' + in_MODELO;
/* Comproba l'existncia del usuari */
IF consulta_USER(in_USR_MOVIL) 
THEN
  /* Afegeix el dispositiu a la taula DISPOSITIVO */
  INSERT INTO UOC.DISPOSITIVO (IMEI, USR_MOVIL, SO_NOMBRE, RES_VALOR, MODELO)
  VALUES (in_IMEI, in_USR_MOVIL, in_SO_NOMBRE, in_RES_VALOR, in_MODELO);
  COMMIT WORK;
  DBMS_OUTPUT.put_line('S''ha donat d''alta el dispositiu segent per a l''usuari amb telfon ' || in_USR_MOVIL || ': ');
  RSP := 'OK';
  IF consulta_DISPO(in_IMEI) 
     THEN
         mostra_DISPO;
         compta_DISPO (USER.USR_MOVIL);
     ELSE
         DBMS_OUTPUT.put_line('Error: No s''ha trobat el dispositiu amb codi IMEI ' || in_IMEI || '.');
         RSP := 'Error: No s''ha trobat el dispositiu amb codi IMEI ' + in_IMEI;
  END IF;
ELSE
    DBMS_OUTPUT.put_line('Error: No es pot donar d''alta el dispositiu.');
    RSP := 'Error: No es pot donar d''alta el dispositiu.';
END IF;

/* Registre de log */
PKG_COMUN.reg_LOG ('alta_DISPO', PARAM_ENTRADA, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('alta_DISPO', PARAM_ENTRADA, RSP);
   END;

END alta_DISPO;
 
/* ----------------------------------------------------- */
/* Procediment Baixa Usuari                              */
/* ----------------------------------------------------- */
/* USAGE : call PKG_USER.baixa_USER (<num.tel>);         */
/* ----------------------------------------------------- */
PROCEDURE baixa_USER (in_USR_MOVIL IN VARCHAR2, RSP OUT VARCHAR2) 
IS

BEGIN  
/* Comproba l'existncia del usuari */
IF consulta_USER(in_USR_MOVIL) 
THEN
   /* Comproba l'existncia del usuari */
   DELETE FROM UOC.USUARIO WHERE (USR_MOVIL = USER.USR_MOVIL);
   COMMIT WORK;
   DBMS_OUTPUT.put_line('S''ha donat de baixa l''usuari amb codi ' || USER.USR_MOVIL || ' i tots els dispositius associats.');
   RSP :='OK';
ELSE
    DBMS_OUTPUT.put_line('Error: No es pot donar de baixa l''usuari.');
    RSP := 'Error: No es pot donar de baixa l''usuari.';
END IF;
/* Compte el nmero d'usuaris donats d'alta */
compta_USER;

/* Registre de log */
PKG_COMUN.reg_LOG ('baixa_USER', in_USR_MOVIL, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('baixa_USER', in_USR_MOVIL, RSP);
   END;

END baixa_USER;

/* ----------------------------------------------------- */
/* Procediment Baixa Dispositiu                          */
/* ----------------------------------------------------- */
/* USAGE : call PKG_USER.baixa_DISPO (<num.tel>, <IMEI>);*/
/* ----------------------------------------------------- */
PROCEDURE baixa_DISPO (in_USR_MOVIL IN VARCHAR2, in_IMEI IN VARCHAR2, RSP OUT VARCHAR2) 
IS
  PARAM_ENTRADA VARCHAR2(500);

BEGIN  
PARAM_ENTRADA := in_USR_MOVIL + ', ' + in_IMEI;
/* Comproba l'existncia del usuari */
IF consulta_USER(in_USR_MOVIL) 
THEN
  /* Comproba l'existncia del dispositiu */
  IF consulta_DISPO(in_IMEI) 
  THEN
     DELETE FROM UOC.DISPOSITIVO WHERE (IMEI = DISPO.IMEI);
     COMMIT WORK;
     DBMS_OUTPUT.put_line('S''ha donat de baixa el dispositiu amb codi IMEI ' || in_IMEI || ' associat a l''usuari amb telfon ' || in_USR_MOVIL || '.');
     RSP :='OK';
  ELSE
     DBMS_OUTPUT.put_line('Error : El dispositiu amb IMEI ' || in_IMEI || ' no est associat a l''usuari amb telfon ' || in_USR_MOVIL || '.');
     RSP :='Error : El dispositiu amb IMEI ' + in_IMEI + ' no est associat a l''usuari amb telfon ' + in_USR_MOVIL;
  END IF;
ELSE
    DBMS_OUTPUT.put_line('Error: No es pot donar de baixa el dispositiu.');
    RSP := 'Error: No es pot donar de baixa el dispositiu.';
END IF;

/* Registre de log */
PKG_COMUN.reg_LOG ('baixa_DISPO', PARAM_ENTRADA, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('baixa_DISPO', PARAM_ENTRADA, RSP);
   END;

END baixa_DISPO;

/* ------------------------------------------------------ */
/* Procediment Modificaci Usuari                         */
/* ------------------------------------------------------ */
/* USAGE : call PKG_USER.modif_USER (<dades usuari>);     */
/* ------------------------------------------------------ */
PROCEDURE modif_USER (in_USR_MOVIL IN VARCHAR2, in_OPERA_NOMBRE IN VARCHAR2, in_PAIS_ID IN CHAR, in_USR_EMAIL IN VARCHAR2, RSP OUT VARCHAR2)
IS
  PARAM_ENTRADA VARCHAR2(500);

BEGIN
PARAM_ENTRADA := in_USR_MOVIL + ', ' + in_OPERA_NOMBRE + ', ' + in_PAIS_ID + ', ' + in_USR_EMAIL;
/* Comproba l'existncia de l'usuari */
IF consulta_USER(in_USR_MOVIL) 
THEN
  /* Modifica l'usuari */
  UPDATE UOC.USUARIO SET OPERA_NOMBRE=in_OPERA_NOMBRE, PAIS_ID=in_PAIS_ID, USR_EMAIL=in_USR_EMAIL;
  COMMIT WORK;
  /* Treu per pantalla les dades de l'usuari modificat */
  DBMS_OUTPUT.put_line('S''han actualitzat les dades de l''usuari: ');
  mostra_USER;
  RSP :='OK';
ELSE
--    DBMS_OUTPUT.put_line('Error: L''usuari amb codi ' || in_USR_MOVIL || ' no existeix.');
    DBMS_OUTPUT.put_line('Error: No es pot modificar l''usuari.');
    RSP :='Error: No es pot modificar l''usuari.';
END IF;

/* Registre de log */
PKG_COMUN.reg_LOG ('modif_USER', PARAM_ENTRADA, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('modif_USER', PARAM_ENTRADA, RSP);
   END;

END modif_USER;

/* ------------------------------------------------------ */
/* Procediment Modificaci Dispositiu                     */
/* ------------------------------------------------------ */
/* USAGE : call PKG_USER.modif_DISPO (<dades usuari>);    */
/* ------------------------------------------------------ */
PROCEDURE modif_DISPO (in_USR_MOVIL IN VARCHAR2, in_IMEI IN VARCHAR2, in_SO_NOMBRE IN VARCHAR2, in_RES_VALOR IN CHAR, in_MODELO IN VARCHAR2, RSP OUT VARCHAR2)
IS
  PARAM_ENTRADA VARCHAR2(500);

BEGIN
PARAM_ENTRADA := in_IMEI + ', ' + in_USR_MOVIL + ', ' + in_SO_NOMBRE + ', ' + in_RES_VALOR + ', ' + in_MODELO;
/* Comproba l'existncia de l'usuari */
IF consulta_USER(in_USR_MOVIL) 
THEN
  /* Comproba l'existncia del dispositiu */
  IF consulta_DISPO(in_IMEI) 
  THEN
     UPDATE UOC.DISPOSITIVO SET SO_NOMBRE=in_SO_NOMBRE, RES_VALOR=in_RES_VALOR, MODELO=in_MODELO;
     COMMIT WORK;
     DBMS_OUTPUT.put_line('S''ha modificat el dispositiu associat a l''usuari amb telfon ' || in_USR_MOVIL || ':');
     mostra_DISPO;
     RSP :='OK';
  ELSE
      DBMS_OUTPUT.put_line('Error: No es pot modificar el dispositiu.');
      RSP :='Error: No es pot modificar el dispositiu.';
  END IF;
ELSE
    DBMS_OUTPUT.put_line('Error: No es pot modificar el dispositiu.');
    RSP := 'Error: No es pot modificar el dispositiu.';
END IF;

/* Registre de log */
PKG_COMUN.reg_LOG ('modif_DISPO', PARAM_ENTRADA, RSP);

/* Control d'Excepcions */
EXCEPTION
   WHEN OTHERS THEN
   BEGIN
        ROLLBACK;
        DBMS_OUTPUT.PUT_LINE('Error : ' || SQLERRM);
        RSP := 'Error : ' + SQLERRM;
        PKG_COMUN.reg_LOG ('modif_DISPO', PARAM_ENTRADA, RSP);
   END;

END modif_DISPO;

END PKG_USER;
/
show errors
