-- +----------------------------------------------------------------------+
-- |                         Jordi Pascual Aran  (JPL)                    |
-- |                         DBA UOC                                      |
-- |----------------------------------------------------------------------|
-- | DATABASE : Oracle                                                    |
-- | FILE     : alim_apliUOC.sql                                          |
-- | CLASS    : Import                                                    |
-- | PURPOSE  : Script de crrega de dades a la taula TMP_APLI            |
-- | DATE     : 29/10/2012                                                |
-- | VERSION  : Oracle 10g                                                |
-- |----------------------------------------------------------------------|
-- | USAGE:  sqlplus <usr>/<passwd> @alim_apliUOC.sql                     |
-- |----------------------------------------------------------------------|

/*  Connexi a Oracle */
connect /as SYSDBA
alter session set current_schema=UOC;

/* Gesti d'errors */
whenever sqlerror exit ROLLBACK
whenever oserror exit ROLLBACK

/* Inicia fitxer log de sortida */
spool '../log/alim_apliUOC.log'
set termout on
prompt
prompt => INICI
prompt

/* Backup taula APLICACION  */
 prompt => 1. Backup de la taula APLICACION
 set termout off 
     @bkup_apliUOC.sql
 set termout on 

spool '../log/alim_apliUOC.log' append
/* Borra registres taula APLICACION  */
prompt
prompt => 3. Borra registres de la taula APLICACION
delete from UOC.APLICACION;

/* Crea taula temporal  */
prompt => 4. Crea taula temporal TMP_APLICACION
CREATE TABLE UOC.TMP_APLICACION   (
   APP_ID             VARCHAR2(15)      not null,
   VERSION            VARCHAR2(20)      not null,
   SO_NOMBRE          VARCHAR2(50)      not null,
   PAIS_ID            CHAR (2)          not null,
   DESA_ID            VARCHAR2(15)      not null,
   RES_VALOR          VARCHAR2(15)      not null,
   APP_BINARIO        BFILE             not null,
   APP_ACTIVA         CHAR(2)           default 'NO' not null,
   FECHA_SUBIDA       DATE              default SYSDATE not null,
   APP_PRECIO         NUMBER (6,2)      not null,           
   APP_DESCRIPCION    VARCHAR2 (1000)   ,        
   APP_PESO           NUMBER (10,3)     ,
   APP_VIDEO          VARCHAR2(100)     ,    
   APP_FICHERO        VARCHAR2(100)     ,    
         constraint CKC_TMP_ACTIVA check (APP_ACTIVA in ('SI','NO'))
) TABLESPACE USERS;

/* Carrega dades a taula temporal  */
prompt => 5. Carrega dades a taula temporal TMP_APLICACION
@load_apliUOC.sql

/* Alimenta taula APLICACION  */
prompt => 6. Alimenta taula APLICACION
insert into UOC.APLICACION (FECHA_SUBIDA, APP_ID, VERSION, PAIS_ID, SO_NOMBRE, RES_VALOR, DESA_ID, APP_ACTIVA, APP_DESCRIPCION, APP_PRECIO, APP_PESO, APP_VIDEO, APP_BINARIO) select FECHA_SUBIDA, APP_ID, VERSION, PAIS_ID, SO_NOMBRE, RES_VALOR, DESA_ID, APP_ACTIVA, APP_DESCRIPCION, APP_PRECIO, APP_PESO, APP_VIDEO, APP_BINARIO from UOC.TMP_APLICACION;

/* Borra taula temporal  */
prompt => 7. Borra taula temporal TMP_APLICACION
set termout off 
    drop table UOC.TMP_APLICACION cascade constraints;
set termout on 

/* Tanca fitxer log de sortida */
prompt
prompt => FINAL : TAULA 'APLICACION' IMPORTADA
prompt

spool off
disconnect;
