-- +----------------------------------------------------------------------+
-- |                         Jordi Pascual Aran  (JPL)                    |
-- |                         DBA UOC                                      |
-- |----------------------------------------------------------------------|
-- | DATABASE : Oracle                                                    |
-- | FILE     : alim_modelUOC.sql                                         |
-- | CLASS    : Import                                                    |
-- | PURPOSE  : Script de crrega de dades a la taula DISPOSITIVO         |
-- | DATE     : 24/10/2012                                                |
-- | VERSION  : Oracle 10g                                                |
-- |----------------------------------------------------------------------|
-- | USAGE:    sqlplus <usr>/<passwd> @alim_modelUOC.sql                  |
-- |----------------------------------------------------------------------|

/*  Connexi a Oracle */
connect /as SYSDBA
alter session set current_schema=UOC;

/* Gesti d'errors */
whenever sqlerror exit ROLLBACK
whenever oserror exit ROLLBACK

/* Inicia fitxer log de sortida */
spool '../log/alim_modelUOC.log'
set termout on
prompt
prompt => INICI
prompt

/* Backup taula MODELO  */
prompt => 1. Backup de la taula MODELO
set termout off 
    @bkup_modelUOC.sql
set termout on 

/* Desactiva Restricci Integritat  */
-- prompt
-- prompt => 2. Desactiva Restriccions Integritat
-- alter table UOC.DISPOSITIVO disable constraint FK_DISPO_MODELO; 

/* Borra registres taula MODELO  */
spool '../log/alim_modelUOC.log' append
prompt
prompt => 3. Borra registres de la taula MODELO
delete from UOC.MODELO;

/* Crea taula temporal  */
prompt => 4. Crea taula temporal TMP_MODELO
CREATE TABLE UOC.TMP_MODELO   (
--   MODELO_ID              NUMBER (3)          not null,
   FABRICANTE             VARCHAR2(50)          not null,
   MODELO                 VARCHAR2(50)          not null
) TABLESPACE USERS;

/* Recrea la seqncia UOC.SEQ_MODELO  */
-- prompt => 5. Recrea la seqncia UOC.SEQ_MODELO
-- DROP SEQUENCE UOC.SEQ_MODELO;
-- CREATE SEQUENCE UOC.SEQ_MODELO
--   START WITH 1
--   MINVALUE 1
--   NOCYCLE
--   NOCACHE;

/* Carrega dades a taula temporal  */
prompt => 6. Carrega dades a taula temporal TMP_MODELO
@load_modelUOC.sql

/* Alimenta taula MODELO  */
prompt => 7. Alimenta taula MODELO
insert into UOC.MODELO (FABRICANTE, MODELO) select FABRICANTE, MODELO from UOC.TMP_MODELO;

/* Borra taula temporal  */
prompt => 8. Borra taula temporal TMP_MODELO
set termout off 
    drop table UOC.TMP_MODELO cascade constraints;
set termout on    

/* Reactiva Restricci Integritat  */
-- prompt
-- prompt => 9. Reactiva Restriccions Integritat
-- alter table UOC.DISPOSITIVO enable constraint FK_DISPO_MODELO;             

/* Tanca fitxer log de sortida */
prompt
prompt => FINAL : TAULA 'MODELO' IMPORTADA
prompt

spool off
disconnect;
