-- +----------------------------------------------------------------------+
-- |                         Jordi Pascual Aran  (JPL)                    |
-- |                         DBA UOC                                      |
-- |----------------------------------------------------------------------|
-- | DATABASE : Oracle                                                    |
-- | FILE     : alim_userUOC.sql                                          |
-- | CLASS    : Import                                                    |
-- | PURPOSE  : Script de crrega de dades a la taula USUARIO             |
-- | DATE     : 4/11/2012                                                 |
-- | VERSION  : Oracle 10g                                                |
-- |----------------------------------------------------------------------|
-- | USAGE:  sqlplus UOC/UOC @alim_userUOC.sql                            |
-- |----------------------------------------------------------------------|

/*  Connexi a Oracle */
connect /as SYSDBA
alter session set current_schema=UOC;

/* Gesti d'errors */
whenever sqlerror exit ROLLBACK
whenever oserror exit ROLLBACK

/* Inicia fitxer log de sortida */
spool '../log/alim_userUOC.log'
set termout on
prompt
prompt => INICI
prompt

/* Backup taules USUARIO i DISPOSITIVO */
prompt => 1. Backup de les taules USUARIO i DISPOSITIVO
set termout off 
    @bkup_userUOC.sql
    @bkup_dispoUOC.sql
set termout on 

/* Buida taules USUARIO i DISPOSITIVO  */
spool '../log/alim_userUOC.log' append
prompt
prompt => 2. Buida taules USUARIO i DISPOSITIVO
delete from UOC.USUARIO;
delete from UOC.DISPOSITIVO;

/* Crea taula temporal */
prompt => 3. Crea taula temporal TMP_USUARIO
CREATE TABLE UOC.TMP_USUARIO   (
   USR_MOVIL            VARCHAR2(40)      not null,
   OPERA_NOMBRE         VARCHAR2(100)     not null,
   PAIS_ID              CHAR (2)          not null,
   IMEI                 VARCHAR2(30)      not null,
   USR_EMAIL            VARCHAR2(100)     ,
   SO_NOMBRE            VARCHAR2(50)      ,
   RES_VALOR            VARCHAR2(50)      ,
   MODELO               VARCHAR2(50)      
) TABLESPACE USERS;

/* Crea taula temporal */
prompt => 4. Crea taula temporal TMP_USUARIO_2
CREATE TABLE UOC.TMP_USUARIO_2   (
   USR_MOVIL            VARCHAR2(40)      not null,
   OPERA_NOMBRE         VARCHAR2(100)     ,
   PAIS_ID              CHAR (2)          ,
   USR_EMAIL            VARCHAR2(100)     
) TABLESPACE USERS;

/* Carrega dades a taula temporal */
prompt => 5. Carrega dades a taula temporal TMP_USUARIO
@load_userUOC.sql

/* Alimenta taula USUARIO */
prompt => 6. Alimenta taula USUARIO
set termout off 
insert into UOC.TMP_USUARIO_2 (USR_MOVIL) select distinct USR_MOVIL from UOC.TMP_USUARIO;
update UOC.TMP_USUARIO_2 b
set (OPERA_NOMBRE, PAIS_ID, USR_EMAIL) = (select distinct OPERA_NOMBRE, PAIS_ID, USR_EMAIL from UOC.TMP_USUARIO a 
 where b.USR_MOVIL = a.USR_MOVIL);
set termout on 
insert into UOC.USUARIO (USR_MOVIL, OPERA_NOMBRE, PAIS_ID, USR_EMAIL) select USR_MOVIL, OPERA_NOMBRE, PAIS_ID, USR_EMAIL from UOC.TMP_USUARIO_2;
commit;
  
/* Alimenta taula DISPOSITIVO */
prompt => 7. Alimenta taula DISPOSITIVO
insert into UOC.DISPOSITIVO (IMEI, USR_MOVIL, SO_NOMBRE, RES_VALOR, MODELO) select IMEI, USR_MOVIL, SO_NOMBRE, RES_VALOR, MODELO from UOC.TMP_USUARIO;

/* Borra taula temporal  */
prompt => 8. Borra taules temporals
set termout off 
    drop table UOC.TMP_USUARIO cascade constraints;
    drop table UOC.TMP_USUARIO_2 cascade constraints;
set termout on 

/* Tanca fitxer log de sortida */
prompt
prompt => FINAL : TAULES 'USUARIO' i 'DISPOSITIVO' IMPORTADES
prompt

spool off
disconnect;
