-- +----------------------------------------------------------------------+
-- |                         Jordi Pascual Aran  (JPL)                    |
-- |                         DBA UOC                                      |
-- |----------------------------------------------------------------------|
-- | DATABASE : Oracle                                                    |
-- | FILE     : crea_fkUOC.sql                                            |
-- | CLASS    : Install                                                   |
-- | PURPOSE  : Script de creaci de claus secundaries                    |
-- | DATE     : 21/10/2012                                                |
-- | VERSION  : Oracle 10g                                                |
-- |----------------------------------------------------------------------|
-- | USAGE:    sqlplus <usr>/<passwd> @crea_fkUOC.sql                     |
-- |----------------------------------------------------------------------|

/* connexi a Oracle      */
connect /as SYSDBA
alter session set current_schema=UOC

/* Gesti d'errors */
-- whenever sqlerror exit ROLLBACK
whenever oserror exit ROLLBACK

/* Inicia fitxer log de sortida */
spool '../log/crea_fkUOC.log'
set termout on
prompt
prompt => INICI
prompt

/* Clau Secundria Taula Aplicaci */
prompt => 1. Restriccions_taula APLICACION
ALTER TABLE UOC.APLICACION   
  ADD (CONSTRAINT FK_APP_DESA foreign key (DESA_ID) references UOC.DESARROLLADOR(DESA_ID) ON DELETE CASCADE,
       CONSTRAINT FK_APP_SO foreign key (SO_NOMBRE) references UOC.SO(SO_NOMBRE),
       CONSTRAINT FK_APP_RES foreign key (RES_VALOR) references UOC.RESOLUCION(RES_VALOR));

/* Clau Secundria Taula Versi */
-- prompt => 2. Restriccions_taula VERSION
-- ALTER TABLE UOC.VERSION   
--   ADD (CONSTRAINT FK_VERSION_APP foreign key (APP_ID, APP_VERSION) references UOC.APLICACION(APP_ID, APP_VERSION),
--        CONSTRAINT FK_VERSION_PAIS foreign key (PAIS_ID) references UOC.PAIS(PAIS_ID),
--        CONSTRAINT FK_VERSION_SO foreign key (SO_NOMBRE) references UOC.SO(SO_NOMBRE));

/* Clau Secundria Taula Desenvolupador */
prompt => 2. Restriccions_taula DESARROLLADOR
ALTER TABLE UOC.DESARROLLADOR   
  ADD (CONSTRAINT FK_DESA_PAISID foreign key (PAIS_ID) references UOC.PAIS(PAIS_ID),
       CONSTRAINT FK_DESA_DIRPAIS foreign key (DIR_PAIS) references UOC.PAIS(PAIS_ID));

/* Clau Secundria Taula Operador */
prompt => 3. Restriccions_taula OPERADOR
ALTER TABLE UOC.OPERADOR   
  ADD (CONSTRAINT FK_OPERA_PAIS foreign key (PAIS_ID) references UOC.PAIS(PAIS_ID));

/* Clau Secundria Taula Usuari */
prompt => 4. Restriccions_taula USUARIO
ALTER TABLE UOC.USUARIO   
  ADD (/*CONSTRAINT FK_USR_IMEI foreign key (IMEI) references UOC.DISPOSITIVO(IMEI) ON DELETE CASCADE,*/
       CONSTRAINT FK_USR_OPERA foreign key (OPERA_NOMBRE) references UOC.OPERADOR(OPERA_NOMBRE),
       CONSTRAINT FK_USR_PAIS foreign key (PAIS_ID) references UOC.PAIS(PAIS_ID));

/* Clau Secundria Taula Dispositiu mbil */
prompt => 5. Restriccions_taula DISPOSITIVO
ALTER TABLE UOC.DISPOSITIVO   
  ADD (CONSTRAINT FK_USR_MOVIL foreign key (USR_MOVIL) references UOC.USUARIO(USR_MOVIL) ON DELETE CASCADE,
       CONSTRAINT FK_DISPO_SO foreign key (SO_NOMBRE) references UOC.SO(SO_NOMBRE),
       CONSTRAINT FK_DISPO_MODELO foreign key (MODELO) references UOC.MODELO(MODELO),
       CONSTRAINT FK_DISPO_RES foreign key (RES_VALOR) references UOC.RESOLUCION(RES_VALOR));

/* Clau Secundria Taula Descrregues */
prompt => 6. Restriccions_taula DESCARGA
ALTER TABLE UOC.DESCARGA   
  ADD (CONSTRAINT FK_DESCARGA_APP foreign key (APP_ID, VERSION) references UOC.APLICACION(APP_ID, VERSION),
       CONSTRAINT FK_DESCARGA_IMEI foreign key (IMEI) references UOC.DISPOSITIVO(IMEI),
       CONSTRAINT FK_DESCARGA_PAIS foreign key (DESCARGA_PAIS) references UOC.PAIS(PAIS_ID),
       CONSTRAINT FK_DESCARGA_PAGO foreign key (PAGO_NOMBRE) references UOC.MODO_PAGO(PAGO_NOMBRE));

-- describe UOC.APLICACION;
-- select * from user_cons_columns;
-- select * from user_constraints;


/* Tanca fitxer log de sortida */
prompt 
prompt => FINAL : CLAUS SECUNDARIES CREADES
prompt

spool off
disconnect;
