-- +----------------------------------------------------------------------+
-- |                         Jordi Pascual Aran  (JPL)                    |
-- |                         DBA UOC                                      |
-- |----------------------------------------------------------------------|
-- | DATABASE : Oracle                                                    |
-- | FILE     : crea_taulaUOC.sql                                         |
-- | CLASS    : Install                                                   |
-- | PURPOSE  : Script de creaci de les taules                           |
-- | DATE     : 16/10/2012                                                |
-- | VERSION  : Oracle 10g                                                |
-- |----------------------------------------------------------------------|
-- | USAGE:    sqlplus <usr>/<passwd> @crea_taulaUOC.sql                  |
-- |----------------------------------------------------------------------|

/* connexi a Oracle */
connect /as SYSDBA
alter session set current_schema=UOC

/* Gesti d'errors */
whenever sqlerror exit ROLLBACK
whenever oserror exit ROLLBACK

/* Inicia fitxer log de sortida */
spool '../log/crea_taulaUOC.log'
set termout on
prompt
prompt => INICI
prompt

/* Taula Sistema Operatiu */
prompt => 1. Crea_taula SO
CREATE TABLE UOC.SO   (
   SO_ID           NUMBER (3)          not null,
   SO_NOMBRE       VARCHAR2(50)        not null
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.SO.SO_ID IS 'Identificador de Sistema Operatiu';
    COMMENT ON COLUMN UOC.SO.SO_NOMBRE IS 'Nom de Sistema Operatiu';
  set termout on

/* Taula Aplicaci */
prompt => 2. Crea_taula APLICACION
CREATE TABLE UOC.APLICACION   (
   APP_ID             VARCHAR2(15)      not null,
   APP_VERSION        VARCHAR2(20)      not null,
   DESA_ID            VARCHAR2(15)      not null,
   RES_ID             NUMBER (3)        not null
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.APLICACION.APP_ID IS 'Identificador aplicaci';
    COMMENT ON COLUMN UOC.APLICACION.APP_VERSION IS 'Versi aplicaci';
    COMMENT ON COLUMN UOC.APLICACION.DESA_ID IS 'Codi desenvolupador(s) aplicaci';
    COMMENT ON COLUMN UOC.APLICACION.RES_ID IS 'Identificador de resoluci mnima';
  set termout on     

/* Taula Versi */
prompt => 3. Crea_taula VERSION
CREATE TABLE UOC.VERSION   (
   APP_ID            VARCHAR2(15)       not null,
   APP_VERSION       VARCHAR2(20)       not null,
   SO_ID             NUMBER (3)         not null,
   PAIS_ID           CHAR (2)           not null,
   APP_BINARIO       BFILE              not null,
   APP_ACTIVA        CHAR(2)            default 'NO' not null,
   FECHA_SUBIDA      DATE               not null default SYSDATE,
   APP_PRECIO        NUMBER (6,2)       not null,           
   APP_DESCRIPCION   VARCHAR2 (1000),        
   APP_PESO          NUMBER (10,3)  ,
   APP_VIDEO         VARCHAR2(100)  ,    
         constraint CKC_ACTIVA check (APP_ACTIVA in ('SI','NO'))
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.VERSION.APP_ID IS 'Identificador aplicaci';
    COMMENT ON COLUMN UOC.VERSION.APP_VERSION IS 'Versi aplicaci';
    COMMENT ON COLUMN UOC.VERSION.SO_ID IS 'Identificador de SO';
    COMMENT ON COLUMN UOC.VERSION.PAIS_ID IS 'Identificador de pas';
    COMMENT ON COLUMN UOC.VERSION.APP_BINARIO IS 'Enlla al fitxer binari per a cada SO';  
    /*INSERT INTO files VALUES (1, BFILENAME('MY_DIR', 'pic1.gif'));*/
    COMMENT ON COLUMN UOC.VERSION.APP_ACTIVA IS 'Aplicaci activa o inactiva';
    COMMENT ON COLUMN UOC.VERSION.FECHA_SUBIDA IS 'Data de pujada aplicaci';
    COMMENT ON COLUMN UOC.VERSION.APP_PRECIO IS 'Preu aplicaci segons pas';
    COMMENT ON COLUMN UOC.VERSION.APP_DESCRIPCION IS 'Descripci aplicaci';
    COMMENT ON COLUMN UOC.VERSION.APP_PESO IS 'Mida aplicaci en Kbits per a cada SO';
    COMMENT ON COLUMN UOC.VERSION.APP_VIDEO IS 'Enlla a video demo per a cada SO';
  set termout on     

/* Taula Resoluci Pantalla */
prompt => 4. Crea_taula RESOLUCION
CREATE TABLE UOC.RESOLUCION   (
   RES_ID              NUMBER (3)           not null,
   RES_VALOR           VARCHAR2(15)         not null
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.RESOLUCION.RES_ID IS 'Identificador de resoluci';
    COMMENT ON COLUMN UOC.RESOLUCION.RES_VALOR IS 'Valor de resoluci';
  set termout on

 /* Taula Desarrollador */
prompt => 5. Crea_taula DESARROLLADOR
CREATE TABLE UOC.DESARROLLADOR   (
   DESA_ID                VARCHAR2(15)       not null,
   DESA_EMPRESA           VARCHAR2(50)       not null,
   PAIS_ID                CHAR(2),
   DESA_REPRESENTANTE     VARCHAR2(50),
   DIR_CALLE              VARCHAR2(200),
   DIR_CP                 VARCHAR2(10),
   DIR_CIUDAD             VARCHAR2(50),
   DIR_PROVINCIA          VARCHAR2(50),
   DIR_PAIS               CHAR(2),
   DESA_TELFPROF          VARCHAR2(40),
   DESA_TELFMOVIL         VARCHAR2(40),
   DESA_TELFPERSO         VARCHAR2(40),
   DESA_FAX               VARCHAR2(40),
   DESA_EMAIL             VARCHAR2(100),
   DESA_WEB               VARCHAR2(200)
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.DESARROLLADOR.DESA_ID IS 'Identificador de desenvolupador';
    COMMENT ON COLUMN UOC.DESARROLLADOR.PAIS_ID IS 'Codi pas registre empresa';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DESA_EMPRESA IS 'Nom empresa';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DESA_REPRESENTANTE IS 'Nom del representant legal';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DIR_CALLE IS 'Adrea oficines centrals';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DIR_CP IS 'Codi Postal oficines centrals';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DIR_CIUDAD IS 'Ciutat oficines centrals';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DIR_PROVINCIA IS 'Provincia oficines centrals';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DIR_PAIS IS 'Codi pas oficines centrals';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DESA_TELFPROF IS 'Telfon profesional desenvolupador';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DESA_TELFMOVIL IS 'Telfon mbil desenvolupador';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DESA_TELFPERSO IS 'Telfon personal desenvolupador';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DESA_FAX IS 'Fax desenvolupador';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DESA_EMAIL IS 'Adrea electrnica desenvolupador';
    COMMENT ON COLUMN UOC.DESARROLLADOR.DESA_WEB IS 'Enlla pgina web desenvolupador';
  set termout on

/* Taula Pas */
prompt => 6. Crea_taula PAIS
CREATE TABLE UOC.PAIS   (
   PAIS_ID              CHAR (2)           not null,
   PAIS_NOMBRE          VARCHAR2(100)       not null
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.PAIS.PAIS_ID IS 'Identificador del pas';
    COMMENT ON COLUMN UOC.PAIS.PAIS_NOMBRE IS 'Nom del pas';
  set termout on

/* Taula Operador */
prompt => 7. Crea_taula OPERADOR
CREATE TABLE UOC.OPERADOR   (
   OPERA_ID             NUMBER (5)          not null,
   OPERA_NOMBRE         VARCHAR2(100)       not null,
   PAIS_ID              CHAR (2),
   OPERA_EMPRESA        VARCHAR2(100)       not null
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.OPERADOR.OPERA_ID IS 'Identificador del Operador';
    COMMENT ON COLUMN UOC.OPERADOR.OPERA_NOMBRE IS 'Nom Operador';
    COMMENT ON COLUMN UOC.OPERADOR.PAIS_ID IS 'Codi Pais Operador';
    COMMENT ON COLUMN UOC.OPERADOR.OPERA_EMPRESA IS 'Nom de empresa global a la que pertanyen el Operador';
  set termout on

/* Taula Usuari */
prompt => 8. Crea_taula USUARIO
CREATE TABLE UOC.USUARIO   (
   USR_MOVIL            VARCHAR2(40)      not null,
   OPERA_ID             NUMBER (5)        not null,
   PAIS_ID              CHAR (2)          not null,
   IMEI                 VARCHAR2(30)      not null,
   USR_EMAIL            VARCHAR2(100)
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.USUARIO.USR_MOVIL IS 'Numro mbil usuari';
    COMMENT ON COLUMN UOC.USUARIO.OPERA_ID IS 'Operador usuari';
    COMMENT ON COLUMN UOC.USUARIO.PAIS_ID IS 'Codi Pais registre usuari';
    COMMENT ON COLUMN UOC.USUARIO.IMEI IS 'Nmero IMEI dispositiu mbil';
    COMMENT ON COLUMN UOC.USUARIO.USR_EMAIL IS 'Adrea electrnica usuari';
  set termout on

/* Taula Dispositiu mbil*/
prompt => 9. Crea_taula DISPOSITIVO
CREATE TABLE UOC.DISPOSITIVO   (
   IMEI                 VARCHAR2(30)      not null,
   SO_ID                NUMBER (3)        not null,
   RES_ID               NUMBER (3)        not null,
   MODELO_ID            NUMBER (5)          
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.DISPOSITIVO.IMEI IS 'Nmero IMEI dispositiu mbil';
    COMMENT ON COLUMN UOC.DISPOSITIVO.SO_ID IS 'Codi sistema operatiu dispositiu mbil';
    COMMENT ON COLUMN UOC.DISPOSITIVO.RES_ID IS 'Codi resoluci pantalla dispositiu mbil';
    COMMENT ON COLUMN UOC.DISPOSITIVO.MODELO_ID IS 'Codi model dispositiu mbil';
  set termout on

/* Taula Mode de Pagament */
prompt => 10. Crea_taula MODO_PAGO
CREATE TABLE UOC.MODO_PAGO   (
   PAGO_ID              NUMBER (3)          not null,
   PAGO_NOMBRE          VARCHAR2(50)        not null
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.MODO_PAGO.PAGO_ID IS 'Identificador del mode de pagament';
    COMMENT ON COLUMN UOC.MODO_PAGO.PAGO_NOMBRE IS 'Descripci del mode de pagament';
  set termout on

 /* Taula Descrregues */
prompt => 11. Crea_taula DESCARGA
CREATE TABLE UOC.DESCARGA   (
   DESCARGA_ID            VARCHAR2(15)       not null,
   APP_ID                 VARCHAR2(15)       not null,
   APP_VERSION            VARCHAR2(20)       not null,
   IMEI                   VARCHAR2(30)       not null,
   PAGO_ID                NUMBER (3)         not null,
   PRECIO                 NUMBER (6,2)       not null,       
   DESCARGA_FECHA         TIMESTAMP          default SYSDATE not null,
   DESCARGA_OPERADOR      NUMBER (5)         not null,
   DESCARGA_PAIS          CHAR (2)           not null
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.DESCARGA.DESCARGA_ID IS 'Identificador de descrrega';
    COMMENT ON COLUMN UOC.DESCARGA.APP_ID IS 'Identificador aplicaci';
    COMMENT ON COLUMN UOC.DESCARGA.APP_VERSION IS 'Versi aplicaci';
    COMMENT ON COLUMN UOC.DESCARGA.IMEI IS 'IMEI dispositiu mbil';
    COMMENT ON COLUMN UOC.DESCARGA.PAGO_ID IS 'Codi del mode de pagament';
    COMMENT ON COLUMN UOC.DESCARGA.PRECIO IS 'Preu aplicaci segons pais usuari';
    COMMENT ON COLUMN UOC.DESCARGA.DESCARGA_FECHA IS 'Data descrrega';
    COMMENT ON COLUMN UOC.DESCARGA.DESCARGA_OPERADOR IS 'Codi operador usuari';
    COMMENT ON COLUMN UOC.DESCARGA.DESCARGA_PAIS IS 'Codi pais de registre usuari';
  set termout on

/* Taula Modelo */
prompt => 12. Crea_taula MODELO
CREATE TABLE UOC.MODELO   (
   MODELO_ID              NUMBER (3)          not null,
   FABRICANTE             VARCHAR2(50)        not null,
   MODELO                 VARCHAR2(50)        not null
) TABLESPACE USERS;
  set termout off
    COMMENT ON COLUMN UOC.MODELO.MODELO_ID IS 'Identificador del model';
    COMMENT ON COLUMN UOC.MODELO.FABRICANTE IS 'Fabricant del dispositiu';
    COMMENT ON COLUMN UOC.MODELO.MODELO IS 'Descripci del model';
  set termout on

/* Tanca fitxer log de sortida */
prompt 
prompt FINAL : TAULES CREADES
prompt

spool off
disconnect;
